from enum import Enum

from pydantic import BaseModel


class SubscriptionProvider(str, Enum):
    hold = "hold"
    subs = "subs"
    vouchers = "vouchers"


class SubscriptionType(str, Enum):
    pro = "pro"
    advanced = "advanced"
    agent = "agent"


class SubscriptionChain(str, Enum):
    base = "base"
    solana = "solana"


class SubscriptionAccount(BaseModel):
    address: str
    chain: SubscriptionChain

    class Config:
        schema_extra = {"example": {"address": "0x0000000000000000000000000000000000000000", "chain": "base"}}


class BaseSubscription(BaseModel):
    id: str
    type: SubscriptionType
    provider: SubscriptionProvider
    started_at: int
    ended_at: int | None
    is_active: bool


class Subscription(BaseSubscription):
    provider_data: dict
    account: SubscriptionAccount
    tags: list[str]


class FetchedSubscription(Subscription):
    post_hash: str


class GetUserSubscriptionsResponse(BaseModel):
    subscriptions: list[BaseSubscription]


class SubscriptionDefinition(BaseModel):
    type: SubscriptionType
    providers: list[SubscriptionProvider]
    multiple: bool = False



import base64

from nacl.exceptions import BadSignatureError
from nacl.signing import VerifyKey
from solders.pubkey import Pubkey


def is_solana_signature_valid(message: str, signature: str, address: str) -> bool:
    """Check if a message signature with a Solana wallet is valid

    Args:
        message: Message that was signed
        signature: base64-encoded message signature
        address: base58-encoded public key
    """
    public_key = Pubkey.from_string(address)
    bytes_message = bytes(message, encoding="utf-8")
    bytes_signature = bytes(signature, "utf-8")

    try:
        retrieved_message = VerifyKey(public_key.__bytes__()).verify(
            smessage=bytes_message, signature=base64.b64decode(bytes_signature)
        )
        return bytes_message == retrieved_message
    except BadSignatureError:
        return False



import base64

from ecies import encrypt as ecies_encrypt, decrypt as ecies_decrypt


def encrypt(data: str, public_key: str | bytes) -> str:
    """Encrypt some data with a public key"""

    encrypted_data = ecies_encrypt(public_key, data.encode())
    # Encoding it in base64 to avoid data loss when stored on Aleph
    base64_encrypted_data = base64.b64encode(encrypted_data).decode()

    return base64_encrypted_data


def decrypt(data: str, private_key: str | bytes) -> str:
    """Decrypt data with a private key"""

    # Decode the base64 data
    encrypted_data = base64.b64decode(data)
    decrypted_data = ecies_decrypt(private_key, encrypted_data).decode()

    return decrypted_data



from base64 import b32encode, b16decode

from typing_extensions import Literal


def get_vm_url(item_hash: str, url_type: Literal["host", "path"] = "host") -> str:
    if url_type == "path":
        return f"https://aleph.sh/vm/{item_hash}"

    # From aleph-client
    hash_base32 = b32encode(b16decode(item_hash.upper())).strip(b"=").lower().decode()
    return f"https://{hash_base32}.aleph.sh"


from pydantic import BaseModel

from libertai_utils.interfaces.subscription import SubscriptionAccount


class BaseDeleteAgentBody(BaseModel):
    subscription_id: str
    password: str


class BaseSetupAgentBody(BaseDeleteAgentBody):
    account: SubscriptionAccount



from eth_account.messages import encode_defunct
from hexbytes import HexBytes
from web3 import Web3


def is_eth_signature_valid(message: str, signature: str, address: str) -> bool:
    """Check if a message signature with an Ethereum wallet is valid"""
    w3 = Web3(Web3.HTTPProvider(""))
    encoded_message = encode_defunct(text=message)
    recovered_address = w3.eth.account.recover_message(
        encoded_message,
        signature=HexBytes(signature),
    )
    return format_eth_address(address) == format_eth_address(recovered_address)


def format_eth_address(address: str) -> str:
    return address.lower()



from libertai_utils.chains.ethereum import is_eth_signature_valid, format_eth_address
from libertai_utils.chains.solana import is_solana_signature_valid
from libertai_utils.interfaces.subscription import SubscriptionChain


def is_signature_valid(chain: SubscriptionChain, message: str, signature: str, address: str) -> bool:
    valid = False

    if chain == SubscriptionChain.base:
        valid = is_eth_signature_valid(message, signature, address)
    elif chain == SubscriptionChain.solana:
        valid = is_solana_signature_valid(message, signature, address)

    return valid


def format_address(address: str, chain: SubscriptionChain) -> str:
    if chain == SubscriptionChain.base:
        return format_eth_address(address)
    return address
